#!/usr/bin/env bash
# ======================================================================
#
# API CLIENT :: GET A CI FILE FROM PACKAGE SERVER
#
# ----------------------------------------------------------------------
# 2021-03-31  v0.0  <axel.hahn@iml.unibe.ch>  init
# ======================================================================

# ----------------------------------------------------------------------
# CONFIG
# ----------------------------------------------------------------------

line="----------------------------------------------------------------------"
bDebug=0

. $0.cfg

# ----------------------------------------------------------------------
# FUNCTIONS
# ----------------------------------------------------------------------

function showhelp(){
echo "SYNTAX:
  -d          enable debug infos
  -e PHASE    phase; overrides env variable IMLCI_PHASE
  -f FILE     filename to get; overrides env variable IMLCI_FILE
  -o OUTFILE  optional output file; default; given filename in current directory
  -p PROJECT  ci project id; overrides env variable IMLCI_PROJECT
  -s SECRET   override secret in IMLCI_PKG_SECRET
  -u URL      URL of iml ci server without trailing /; overrides env variable IMLCI_URL

DEFAULTS:
  You don't need to set all values by command line. Use a config to set defaults
  $0.cfg

EXAMPLES:
  `basename $0` -u https://ci.example.com -f [FILE]
"
}


function makeRequest(){

  local apiMethod=$1
  local apiRequest=$2
  local outfile=$3
  local secret=$4

  # local outfile=$( mktemp )

  echo $line
  echo $apiMethod ${apiHost}${apiRequest}
  echo $line

  if [ ! -z "$secret" ]; then

    # --- date in http format
    LANG=en_EN
    # export TZ=GMT
    apiTS=`date "+%a, %d %b %Y %H:%M:%S %Z"`


# --- generate data to hash: method + uri + timestamp; delimited with line break
data="${apiMethod}
${apiRequest}
${apiTS}
"

    # generate hash - split in 2 commands (piping "cut" sends additional line break)
    myHash=`echo -n "$data" | openssl sha1 -hmac "${secret}" | cut -f 2 -d" "`
    myHash=`echo -n "$myHash" | base64`

    curl\
      -H "Accept: application/json" -H "Content-Type: application/json" \
      -H "Date: ${apiTS}" \
      -H "Authorization: demo-bash-client:${myHash}" \
      -X $apiMethod \
      -o "${outfile}.downloading" \
      --fail \
      ${IMLCI_URL}${apiRequest}
    if [ $? -eq 0 ]; then
        echo Download OK.
        mv "${outfile}.downloading" "${outfile}"
        ls -l "${outfile}"
    else
        echo ERROR: Download failed.
        # rm -f "${outfile}.downloading"
    fi
  else
    curl\
      -H "Accept: application/json" -H "Content-Type: application/json" \
      -X $apiMethod \
      -o "${outfile}.downloading" \
      ${IMLCI_URL}${apiRequest}
  fi

}


# ----------------------------------------------------------------------
# MAIN
# ----------------------------------------------------------------------

echo
echo ===== CIPGK GETTER :: `date` =====
echo

if  [ $# -lt 1 ]; then
  showhelp
  exit 1
fi




while getopts "de:f:o:p:s:u:" option; do 
    case ${option} in
      d) bDebug=1 ;;
      e) export IMLCI_PHASE=$OPTARG ;;
      f) export IMLCI_FILE=$OPTARG ;;
      o) export IMLCI_OUTFILE=$OPTARG ;;
      p) export IMLCI_PROJECT=$OPTARG ;;
      s) export IMLCI_PKG_SECRET=$OPTARG ;;
      u) export IMLCI_URL=$OPTARG ;;
      *)
        echo ERROR: invalid option [${option}]
        exit 2
    esac
done

test -z ${IMLCI_OUTFILE} && IMLCI_OUTFILE=$IMLCI_FILE

if [ $bDebug = 1 ]; then
    echo $line
    echo
    echo DEBUG INFOS
    echo
    echo ----- content of $0.cfg
    cat $0.cfg 2>/dev/null
    echo
    echo '----- Params (override config values)'
    echo $*
    echo
    echo ----- used values
    echo "IMLCI_URL = $IMLCI_URL"
    echo "IMLCI_PKG_SECRET = $IMLCI_PKG_SECRET"
    echo "IMLCI_PROJECT = $IMLCI_PROJECT"
    echo "IMLCI_PHASE = $IMLCI_PHASE"
    echo "IMLCI_FILE = $IMLCI_FILE"
    echo "IMLCI_OUTFILE = $IMLCI_OUTFILE"

    echo
fi

makeRequest GET "/packages/$IMLCI_PHASE/$IMLCI_PROJECT/$IMLCI_FILE" "$IMLCI_OUTFILE" "$IMLCI_PKG_SECRET"
